!=========================================================================
! Copyright (C) GemTalk Systems 2013-2017.  All Rights Reserved.
!=========================================================================

! ------------------- Class definitions 

expectvalue %String
doit
(ReadStreamPortable subclass: 'PPStream'
  instVarNames: #(newlines)
  classVars: #()
  classInstVars: #()
  poolDictionaries: #()
  inDictionary: Globals
  options: #()
) definition.
%

expectvalue %Boolean
! ------------------- Class comment for PPStream
doit
PPStream comment: 
'A positional stream implementation used for parsing. It overrides some methods for optimization reasons.'.
true
%
! ------------------- Class methods 

! Remove existing behavior from PPStream
removeallmethods PPStream
removeallclassmethods PPStream
! ------------------- Class methods for PPStream
! ------------------- Instance methods for PPStream
category: 'converting'
set compile_env: 0
method: PPStream
asPetitStream
	^ self
%
category: 'accessing'
set compile_env: 0
method: PPStream
back
	"Go back one element and return it."

	self skip: -1.
	^ self peek
%
category: 'accessing'
set compile_env: 0
method: PPStream
collection
	"Answer the underlying collection."
	
	^ collection
%
category: 'positioning'
set compile_env: 0
method: PPStream
column
	^ self column: position.
%
category: 'positioning'
set compile_env: 0
method: PPStream
column: pos
	| nl |
	(pos = -1) ifTrue: [  ^ 0 ].
	(pos > readLimit) ifTrue: [ ^ self error: 'Out of limit' ].
	
	nl := self newlines.
	nl keysAndValuesDo: [ :index :value |
		(value > pos) ifTrue: [ ^ pos - (nl at: (index - 1)) + 1]
	].	

	^ pos - (nl at: (nl size )) + 1
%
category: 'positioning'
set compile_env: 0
method: PPStream
fillNewlines
	| tmp line |
	newlines := OrderedCollection new.	
	
	tmp := position.
	line := 0.
	
	(0 to: readLimit) do: [:index |
		position := index.
		self isStartOfLine ifTrue: [ newlines add: position ]
	].
	position := tmp.
	newlines := newlines asArray.
	^ newlines
%
category: 'queries'
set compile_env: 0
method: PPStream
insideCRLF
	(position < 1) ifTrue: [ ^ false ].
	
	^ (self peek = (Character codePoint: 10)) and: [ self peekBack = (Character codePoint: 13) ]
%
category: 'queries'
set compile_env: 0
method: PPStream
isEndOfLine
	self atEnd ifTrue: [ ^ true ].
	self insideCRLF ifTrue: [ ^ false ].
	^ (self peek = (Character codePoint: 13) or: [ self peek = (Character codePoint: 10)]).
%
category: 'queries'
set compile_env: 0
method: PPStream
isStartOfLine
	(position = 0) ifTrue: [ ^ true ].

	self insideCRLF ifTrue: [ ^ false ].
	
	^ (self peekBack = (Character codePoint: 13)) or: [ self peekBack = (Character codePoint: 10)].
%
category: 'positioning'
set compile_env: 0
method: PPStream
line
	^ self line: position
%
category: 'positioning'
set compile_env: 0
method: PPStream
line: pos
	| nl |
	(pos = -1) ifTrue: [  ^ 0 ].
	(pos > readLimit) ifTrue: [ ^ self error: 'Out of limit' ].
	
	nl := self newlines.
	nl keysAndValuesDo: [ :index :value |
		(value > pos) ifTrue: [ ^ (index - 1)]
	].	

	^ nl size
%
category: 'positioning'
set compile_env: 0
method: PPStream
newlines
	^ newlines ifNil: [ 
		newlines := self fillNewlines.
	]
%
category: 'accessing'
set compile_env: 0
method: PPStream
next: anInteger 
	"Answer up to anInteger elements of my collection. Overridden for efficiency."

	| answer endPosition |
	endPosition := position + anInteger min: readLimit.
	answer := collection copyFrom: position + 1 to: endPosition.
	position := endPosition.
	^ answer
%
category: 'accessing'
set compile_env: 0
method: PPStream
peek
	"An improved version of peek, that is slightly faster than the built in version."

	^ self atEnd ifFalse: [ collection at: position + 1 ]
%
category: 'accessing'
set compile_env: 0
method: PPStream
peekBack
	"Return the element at the previous position, without changing position."

	| element |
	element := self back.
	self skip: 1.
	^ element
%
category: 'accessing'
set compile_env: 0
method: PPStream
position: anInteger
	"The receiver does not check for invalid arguments passed to this method, as it is solely used with valid indexes for backtracking."

	position := anInteger
%
category: 'printing'
set compile_env: 0
method: PPStream
printOn: aStream
	(collection isKindOf: CharacterCollection)
		ifFalse: [ ^ super printOn: aStream ].
	aStream
		nextPutAll: (collection copyFrom: 1 to: position);
		nextPut: (Character codePoint: 183);
		nextPutAll: (collection copyFrom: position + 1 to: readLimit)
%
category: 'accessing'
set compile_env: 0
method: PPStream
size
	"
		The same implementation as a ReadStream. Implemented here for compatibility with Smalltalk/X
		that has different implementation in a ReadStream
	"
	^readLimit
%
category: 'accessing'
set compile_env: 0
method: PPStream
uncheckedPeek
	"An unchecked version of peek that throws an error if we try to peek over the end of the stream, even faster than #peek."

	^ collection at: position + 1
%
expectValue %Boolean
doit
PPStream category: 'PetitParser-Core'.
true
%
! ------------------- Class initializers 
